% fig3.m
% (sym_trend.m)
% Plots the ratio of the energy in the asymmetric part to
% the symmetric part of the solution.
% The independent variable is the TEQ length.
%
% Unit norm TEQ (UNT) results are also included.
% They were studied in a EURASIP JASP paper.
% See paper ``J05."
%
% Written by Rick Martin, 07/31/2002
% Last updated 08/26/2002
% Cornell University Blind Equalization Research Group

% system parameters
Lws = [3:1:240]-1;   % equalizer length - 1
%Lws = [3:1:50]-1;    % (use this line for debugging)
nu = 32;             % cyclic prefix length
Nlw = length(Lws);   % number of TEQ lengths to examine
Ncsa = 8;            % average over CSA loops 1 to Ncsa

% allocate memory
Rmel = zeros(Nlw,Ncsa);
Runt = zeros(Nlw,Ncsa);

% loop through channels and TEQ lengths
hh = waitbar(0,'Computing many TEQs...');
for j=1:Ncsa
   j
   eval(['load ../channels/csaloop',num2str(j),'.time;']);
   eval(['h = (csaloop',num2str(j),'(:,2));']);
   h = h/norm(h);       % unit norm channel
   Lh = length(h)-1;    % channel length - 1
   for i=1:Nlw
      Lw = Lws(i);      % equalizer length - 1
      Lc = Lw+Lh;       % length of effective channel - 1

      % maximum shortening SNR solution
      % algorithm proposed by Melsa, Younce, and Rohrs, Dec. 1996
      [wmelsa,dopt_mel] = mssnr(h,nu+1,Lw+1,0,Lc-nu);
      delta = dopt_mel;   % pick UNT delay for fair comparison

      % MSSNR solution with unit norm TEQ (w_unt)
      H = convmtx(h,Lw+1);
      Hwin = H([delta+1:delta+nu+1],:);
      Hwall = H([1:delta,delta+nu+2:end],:);
      A = Hwall'*Hwall;
      B = Hwin'*Hwin;
      [V,D] = eig(A);
      [dummy,index] = min(abs(diag(D)));
      w_unt = V(:,index);

      % decompose each TEQ into a symmetric and an asymmetric part
      [w_unt_s,w_unt_a] = symasym(w_unt);
      [w_mel_s,w_mel_a] = symasym(wmelsa);

      % find Energy ratios of asymmetric to symmetric part
      Rmel(i,j) = (norm(w_mel_a)/norm(w_mel_s))^2;
      Runt(i,j) = (norm(w_unt_a)/norm(w_unt_s))^2;

      waitbar( ((j-1)*Nlw+i)/(Ncsa*Nlw), hh);
   end
end
close(hh);

aRmel = mean(Rmel,2);
aRunt = mean(Runt,2);

figure(1);
clf; set(0,'DefaultAxesFontSize',16);
plot(Lws+1,aRunt,'r-');
xlabel('length of TEQ','FontSize',16);
ylabel('||w_{skew}||^2/||w_{sym}||^2','FontSize',16);
axis([0 240 0 0.15]);
grid on;

figure(2);
clf; set(0,'DefaultAxesFontSize',16);
plot(Lws+1,aRmel,'b-', Lws+1,aRunt,'r--');
xlabel('length of TEQ','FontSize',16);
ylabel('||w_{skew}||^2/||w_{sym}||^2','FontSize',16);
legend('MSSNR','MSSNR-UNT');
grid on;

