function [wopt,bopt,dopt,MMSE] = mmse_sym(h,Nb,Nw,Dmin,Dmax,ssx,ssn)
% [wopt,bopt,dopt,MMSE] = mmse_sym(h,Nb,Nw,Dmin,Dmax,ssx,ssn)
% Computes the MMSE TEQ under white signal and noise assumptions,
% using a symmetric TIR
%
% wopt = delay-optimized MMSE TEQ
% bopt = delay-optimized MMSE TIR (symmetric)
% dopt = optimal delay (counting starts at zero)
% MMSE = MMSE obtained using wopt, bopt, and dopt
% h = channel
% Nb = TIR length
% Nw = TEQ length
% Dmin = minimum delay, 0 or larger
% Dmax = maximum delay, at most length(h)+Nw-Nb-1
% ssx = variance of white input signal
% ssn = variance of white noise signal
%
% Written by Rick Martin, 05/??/2003
% Last updated on 05/22/2003
% Cornell University Blind Equalization Research Group

% intialize variables and perform some error checks
h = h(:);
Dmaxtemp = length(h)+Nw-Nb;
Dmin = min(max(0,Dmin),Dmaxtemp);
Dmax = min(max(Dmin,Dmax),Dmaxtemp);
if mod(Nb,2)==0, error(' TIR length should be odd! '); end;
Nbhalf = floor(Nb/2);
MMSE = inf;
H = convmtx(h,Nw);
HHi = pinv( toeplitz(H.' * H(:,1)) + ssn*eye(Nw) );

for delay = Dmin:Dmax % for each delay to be searched
   Hwin = H(delay+1:delay+Nb,:);
   if delay==Dmin
      Rdelta = eye(Nb) - Hwin*HHi*Hwin.';
   else
      Rdelta(1:Nb-1,1:Nb-1) = Rdelta(2:Nb,2:Nb);
      temp = zeros(Nb,1);
      temp(Nb) = 1;
      temp = temp - Hwin*( HHi * (Hwin(Nb,:)).' );
      Rdelta(:,Nb) = temp;   
      Rdelta(Nb,:) = temp.';
   end
   Rhat = zeros(Nbhalf+1);
   Rhat(1:Nbhalf,1:Nbhalf) = (1/2)*( ...
      Rdelta(1:Nbhalf,1:Nbhalf) + ...
      fliplr(Rdelta(1:Nbhalf,Nbhalf+2:Nb)) + ...
      flipud(Rdelta(Nbhalf+2:Nb,1:Nbhalf)) + ...
      fliplr(flipud( Rdelta(Nbhalf+2:Nb,Nbhalf+2:Nb) )) );
   temp = (1/sqrt(2)) * ( ...
      Rdelta(1:Nbhalf,Nbhalf+1) + ...
      flipud(Rdelta(Nbhalf+2:Nb,Nbhalf+1)) );
   Rhat(1:Nbhalf,Nbhalf+1) = temp;
   Rhat(Nbhalf+1,1:Nbhalf) = temp.';
   Rhat(Nbhalf+1,Nbhalf+1) = Rdelta(Nbhalf+1,Nbhalf+1);

   [evecs,evals] = eig(Rhat);
   [lambda,ind] = min(diag(evals));   

   if lambda < MMSE
      % save the TIR, TEQ, delay, and MMSE
      vhat = evecs(:,ind);
      v = (1/sqrt(2)) * vhat(1:Nbhalf);
      gamma = vhat(Nbhalf+1);
      bopt = [v; gamma; flipud(v)];
      wopt = HHi * (Hwin.' * bopt);
      dopt = delay;
      MMSE = lambda;
   end
end
